==> codeberg-cli-0.4.0.tar.gz: codeberg-cli/src/actions/pull_request/view.rs <==
use crate::cli::pull_request::view::ViewPullRequestsArgs;
use crate::git_info::repo_owner::RepoAndOwner;
use crate::render::comment::render_comment;
use crate::render::datetime::render_datetime_and_info;
use crate::render::option::option_display;
use crate::render::spinner::spin_until_ready;
use crate::render::ui::fuzzy_select_with_key;
use crate::types::api::state_type::StateType;
use crate::types::context::BergContext;
use anyhow::Context;
use forgejo_api::structs::{
    IssueGetCommentsQuery, PullRequest, RepoListPullRequestsQuery, RepoListPullRequestsQueryState,
};
use itertools::Itertools;

use crate::actions::text_manipulation::select_prompt_for;

use super::display_pull_request;

pub async fn view_pull(ctx: BergContext<ViewPullRequestsArgs>) -> anyhow::Result<()> {
    let RepoAndOwner { repo, owner } = ctx.repo_owner()?;
    let state = match ctx.args.state {
        StateType::Closed => RepoListPullRequestsQueryState::Closed,
        StateType::Open => RepoListPullRequestsQueryState::Open,
        StateType::All => RepoListPullRequestsQueryState::All,
    };
    let pull_requests_list = spin_until_ready(ctx.client.repo_list_pull_requests(
        owner.as_str(),
        repo.as_str(),
        RepoListPullRequestsQuery {
            state: Some(state),
            ..Default::default()
        },
    ))
    .await?;

    let pull_request = fuzzy_select_with_key(
        &pull_requests_list,
        select_prompt_for("pull request"),
        display_pull_request,
    )?;

    if ctx.args.comments {
        spin_until_ready(present_pull_request_comments(&ctx, pull_request)).await?;
    } else {
        present_pull_request_overview(&ctx, pull_request);
    }

    Ok(())
}

fn present_pull_request_overview(
    ctx: &BergContext<ViewPullRequestsArgs>,
    pull_request: &PullRequest,
) {
    let rendered_datetime = pull_request
        .created_at
        .as_ref()
        .map(render_datetime_and_info)
        .unwrap_or_else(|| String::from("?"));

    let mut table = ctx.make_table();

    table
        .set_header(vec![format!(
            "Pull Request #{}",
            option_display(&pull_request.id)
        )])
        .add_row(vec![
            String::from("Title"),
            option_display(&pull_request.title),
        ])
        .add_row(vec![String::from("Created"), rendered_datetime])
        .add_row(vec![
            String::from("Labels"),
            pull_request
                .labels
                .iter()
                .flatten()
                .map(|label| option_display(&label.name))
                .join(", "),
        ])
        .add_row(vec![
            String::from("Description"),
            option_display(&pull_request.body),
        ]);

    println!("{table}");
}

async fn present_pull_request_comments(
    ctx: &BergContext<ViewPullRequestsArgs>,
    pull_request: &PullRequest,
) -> anyhow::Result<()> {
    let RepoAndOwner { repo, owner } = ctx.repo_owner()?;
    let comments = ctx
        .client
        .issue_get_comments(
            owner.as_str(),
            repo.as_str(),
            pull_request.id.context("Selected pull request has no ID")?,
            IssueGetCommentsQuery::default(),
        )
        .await?;
    let header = format!(
        "Pull Request #{} {}",
        option_display(&pull_request.id),
        if comments.is_empty() {
            "(no comments)"
        } else {
            "comments"
        }
    );

    let mut table = ctx.make_table();

    table
        .set_header(vec![header])
        .add_rows(comments.into_iter().filter_map(|comment| {
            let username = comment.user.as_ref()?.login.as_ref()?.as_str();
            let creation_time = comment.created_at.as_ref()?;
            let comment = comment.body.as_ref()?;
            let comment = render_comment(&ctx.config, username, creation_time, comment);
            Some(vec![comment])
        }));

    println!("{table}");

    Ok(())
}

==> codeberg-cli-0.4.1.tar.gz: codeberg-cli/src/actions/pull_request/view.rs <==
use super::display_pull_request;
use crate::actions::{text_manipulation::select_prompt_for, GeneralArgs};
use crate::render::comment::render_comment;
use crate::render::datetime::render_datetime_and_info;
use crate::render::option::option_display;
use crate::render::spinner::spin_until_ready;
use crate::render::ui::fuzzy_select_with_key;
use crate::types::api::state_type::StateType;
use crate::types::context::BergContext;
use crate::types::git::OwnerRepo;
use anyhow::Context;
use clap::Parser;
use forgejo_api::structs::{
    IssueGetCommentsQuery, PullRequest, RepoListPullRequestsQuery, RepoListPullRequestsQueryState,
};
use itertools::Itertools;

#[derive(Parser, Debug)]
#[command(about = "View details of a selected pull request")]
pub struct ViewPullRequestsArgs {
    /// Select from pull requests with the chosen state
    #[arg(
        short,
        long,
        value_enum,
        default_value_t = StateType::All,
    )]
    pub state: StateType,

    /// Disabled: display issue summary | Enabled: display issue comment history
    #[arg(short, long)]
    pub comments: bool,
}

impl ViewPullRequestsArgs {
    pub async fn run(self, general_args: GeneralArgs) -> anyhow::Result<()> {
        let _ = general_args;
        let ctx = BergContext::new(self).await?;

        let OwnerRepo { repo, owner } = ctx.owner_repo()?;
        let state = match ctx.args.state {
            StateType::Closed => RepoListPullRequestsQueryState::Closed,
            StateType::Open => RepoListPullRequestsQueryState::Open,
            StateType::All => RepoListPullRequestsQueryState::All,
        };
        let pull_requests_list = spin_until_ready(ctx.client.repo_list_pull_requests(
            owner.as_str(),
            repo.as_str(),
            RepoListPullRequestsQuery {
                state: Some(state),
                ..Default::default()
            },
        ))
        .await?;

        let pull_request = fuzzy_select_with_key(
            &pull_requests_list,
            select_prompt_for("pull request"),
            display_pull_request,
        )?;

        if ctx.args.comments {
            spin_until_ready(present_pull_request_comments(&ctx, pull_request)).await?;
        } else {
            present_pull_request_overview(&ctx, pull_request);
        }

        Ok(())
    }
}

fn present_pull_request_overview(
    ctx: &BergContext<ViewPullRequestsArgs>,
    pull_request: &PullRequest,
) {
    let rendered_datetime = pull_request
        .created_at
        .as_ref()
        .map(render_datetime_and_info)
        .unwrap_or_else(|| String::from("?"));

    let mut table = ctx.make_table();

    table
        .set_header(vec![format!(
            "Pull Request #{}",
            option_display(&pull_request.id)
        )])
        .add_row(vec![
            String::from("Title"),
            option_display(&pull_request.title),
        ])
        .add_row(vec![String::from("Created"), rendered_datetime])
        .add_row(vec![
            String::from("Labels"),
            pull_request
                .labels
                .iter()
                .flatten()
                .map(|label| option_display(&label.name))
                .join(", "),
        ])
        .add_row(vec![
            String::from("Description"),
            option_display(&pull_request.body),
        ]);

    println!("{table}");
}

async fn present_pull_request_comments(
    ctx: &BergContext<ViewPullRequestsArgs>,
    pull_request: &PullRequest,
) -> anyhow::Result<()> {
    let OwnerRepo { repo, owner } = ctx.owner_repo()?;
    let comments = ctx
        .client
        .issue_get_comments(
            owner.as_str(),
            repo.as_str(),
            pull_request.id.context("Selected pull request has no ID")?,
            IssueGetCommentsQuery::default(),
        )
        .await?;
    let header = format!(
        "Pull Request #{} {}",
        option_display(&pull_request.id),
        if comments.is_empty() {
            "(no comments)"
        } else {
            "comments"
        }
    );

    let mut table = ctx.make_table();

    table
        .set_header(vec![header])
        .add_rows(comments.into_iter().filter_map(|comment| {
            let username = comment.user.as_ref()?.login.as_ref()?.as_str();
            let creation_time = comment.created_at.as_ref()?;
            let comment = comment.body.as_ref()?;
            let comment = render_comment(&ctx.config, username, creation_time, comment);
            Some(vec![comment])
        }));

    println!("{table}");

    Ok(())
}
